var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  __markAsModule(target);
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && key !== "default")
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toModule = (module2) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", module2 && module2.__esModule && "default" in module2 ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
__export(exports, {
  createDefaultServer: () => createDefaultServer
});
var AWS = __toModule(require("aws-sdk"));
var import_services = __toModule(require("../services"));
var import_cognitoService = __toModule(require("../services/cognitoService"));
var import_cache = __toModule(require("../services/dataStore/cache"));
var import_stormDb = __toModule(require("../services/dataStore/stormDb"));
var import_consoleMessageSender = __toModule(require("../services/messageDelivery/consoleMessageSender"));
var import_messageDelivery = __toModule(require("../services/messageDelivery/messageDelivery"));
var import_otp = __toModule(require("../services/otp"));
var import_tokenGenerator = __toModule(require("../services/tokenGenerator"));
var import_userPoolService = __toModule(require("../services/userPoolService"));
var import_Router = __toModule(require("./Router"));
var import_config = __toModule(require("./config"));
var import_server = __toModule(require("./server"));
var import_crypto = __toModule(require("../services/crypto"));
const createDefaultServer = async (logger) => {
  const configDirectory = ".cognito";
  const dataDirectory = `${configDirectory}/db`;
  const ctx = {
    logger
  };
  const config = await (0, import_config.loadConfig)(ctx, new import_stormDb.StormDBDataStoreFactory(configDirectory, new import_cache.InMemoryCache()));
  logger.debug({ config }, "Loaded config");
  const clock = new import_services.DateClock();
  const dataStoreFactory = new import_stormDb.StormDBDataStoreFactory(dataDirectory, new import_cache.InMemoryCache());
  const cognitoServiceFactory = new import_cognitoService.CognitoServiceFactoryImpl(dataDirectory, clock, dataStoreFactory, new import_userPoolService.UserPoolServiceFactoryImpl(clock, dataStoreFactory));
  const cognitoClient = await cognitoServiceFactory.create(ctx, config.UserPoolDefaults);
  const triggers = new import_services.TriggersService(clock, cognitoClient, new import_services.LambdaService(config.TriggerFunctions, new AWS.Lambda(config.LambdaClient)), new import_crypto.CryptoService(config.KMSConfig));
  return (0, import_server.createServer)((0, import_Router.Router)({
    clock,
    cognito: cognitoClient,
    config,
    messages: new import_services.MessagesService(triggers, new import_messageDelivery.MessageDeliveryService(new import_consoleMessageSender.ConsoleMessageSender())),
    otp: import_otp.otp,
    tokenGenerator: new import_tokenGenerator.JwtTokenGenerator(clock, triggers, config.TokenConfig),
    triggers
  }), logger, {
    development: !!process.env.COGNITO_LOCAL_DEVMODE
  });
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  createDefaultServer
});
