var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  __markAsModule(target);
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && key !== "default")
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toModule = (module2) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", module2 && module2.__esModule && "default" in module2 ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
__export(exports, {
  JwtTokenGenerator: () => JwtTokenGenerator
});
var import_jsonwebtoken = __toModule(require("jsonwebtoken"));
var uuid = __toModule(require("uuid"));
var import_cognitoLocal_private = __toModule(require("../keys/cognitoLocal.private.json"));
var import_userPoolService = __toModule(require("./userPoolService"));
const RESERVED_CLAIMS = [
  "acr",
  "amr",
  "aud",
  "at_hash",
  "auth_time",
  "azp",
  "cognito:username",
  "exp",
  "iat",
  "identities",
  "iss",
  "jti",
  "nbf",
  "nonce",
  "origin_jti",
  "sub",
  "token_use"
];
const applyTokenOverrides = (token, overrides) => {
  const claimsToSuppress = ((overrides == null ? void 0 : overrides.claimsToSuppress) ?? []).filter((claim) => !RESERVED_CLAIMS.includes(claim));
  const claimsToOverride = Object.entries((overrides == null ? void 0 : overrides.claimsToAddOrOverride) ?? []).filter(([claim]) => !RESERVED_CLAIMS.includes(claim));
  return Object.fromEntries([...Object.entries(token), ...claimsToOverride].filter(([claim]) => !claimsToSuppress.includes(claim)));
};
const formatExpiration = (duration, unit, fallback) => duration ? `${duration}${unit}` : fallback;
class JwtTokenGenerator {
  constructor(clock, triggers, tokenConfig) {
    this.clock = clock;
    this.triggers = triggers;
    this.tokenConfig = tokenConfig;
  }
  async generate(ctx, user, userGroups, userPoolClient, clientMetadata, source) {
    var _a, _b, _c;
    const eventId = uuid.v4();
    const authTime = Math.floor(this.clock.get().getTime() / 1e3);
    const sub = (0, import_userPoolService.attributeValue)("sub", user.Attributes);
    const accessToken = {
      auth_time: authTime,
      client_id: userPoolClient.ClientId,
      event_id: eventId,
      iat: authTime,
      jti: uuid.v4(),
      scope: "aws.cognito.signin.user.admin",
      sub,
      token_use: "access",
      username: user.Username
    };
    let idToken = __spreadValues({
      "cognito:username": user.Username,
      auth_time: authTime,
      email: (0, import_userPoolService.attributeValue)("email", user.Attributes),
      email_verified: Boolean((0, import_userPoolService.attributeValue)("email_verified", user.Attributes) ?? false),
      event_id: eventId,
      iat: authTime,
      jti: uuid.v4(),
      sub,
      token_use: "id"
    }, (0, import_userPoolService.attributesToRecord)((0, import_userPoolService.customAttributes)(user.Attributes)));
    if (userGroups.length) {
      accessToken["cognito:groups"] = userGroups;
      idToken["cognito:groups"] = userGroups;
    }
    if (this.triggers.enabled("PreTokenGeneration")) {
      const result = await this.triggers.preTokenGeneration(ctx, {
        clientId: userPoolClient.ClientId,
        clientMetadata,
        source,
        userAttributes: user.Attributes,
        username: user.Username,
        groupConfiguration: {
          groupsToOverride: void 0,
          iamRolesToOverride: void 0,
          preferredRole: void 0
        },
        userPoolId: userPoolClient.UserPoolId
      });
      idToken = applyTokenOverrides(idToken, result.claimsOverrideDetails);
    }
    const issuer = `${this.tokenConfig.IssuerDomain}/${userPoolClient.UserPoolId}`;
    return {
      AccessToken: import_jsonwebtoken.default.sign(accessToken, import_cognitoLocal_private.default.pem, {
        algorithm: "RS256",
        issuer,
        expiresIn: formatExpiration(userPoolClient.AccessTokenValidity, ((_a = userPoolClient.TokenValidityUnits) == null ? void 0 : _a.AccessToken) ?? "hours", "24h"),
        keyid: "CognitoLocal"
      }),
      IdToken: import_jsonwebtoken.default.sign(idToken, import_cognitoLocal_private.default.pem, {
        algorithm: "RS256",
        issuer,
        expiresIn: formatExpiration(userPoolClient.IdTokenValidity, ((_b = userPoolClient.TokenValidityUnits) == null ? void 0 : _b.IdToken) ?? "hours", "24h"),
        audience: userPoolClient.ClientId,
        keyid: "CognitoLocal"
      }),
      RefreshToken: import_jsonwebtoken.default.sign({
        "cognito:username": user.Username,
        email: (0, import_userPoolService.attributeValue)("email", user.Attributes),
        iat: authTime,
        jti: uuid.v4()
      }, import_cognitoLocal_private.default.pem, {
        algorithm: "RS256",
        issuer,
        expiresIn: formatExpiration(userPoolClient.RefreshTokenValidity, ((_c = userPoolClient.TokenValidityUnits) == null ? void 0 : _c.RefreshToken) ?? "days", "7d")
      })
    };
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  JwtTokenGenerator
});
