import { AttributeListType } from "aws-sdk/clients/cognitoidentityserviceprovider";
import { Clock } from "../clock";
import { CognitoService } from "../cognitoService";
import { Lambda } from "../lambda";
import { User } from "../userPoolService";
import { Trigger } from "./trigger";
export declare type UserMigrationTrigger = Trigger<{
    userPoolId: string;
    clientId: string;
    username: string;
    password: string;
    userAttributes: AttributeListType;
    /**
     * One or more key-value pairs that you can provide as custom input to the Lambda function that you specify for the
     * migrate user trigger. You can pass this data to your Lambda function by using the ClientMetadata parameter in the
     * AdminRespondToAuthChallenge and ForgotPassword API actions.
     *
     * Source: https://docs.aws.amazon.com/cognito/latest/developerguide/user-pool-lambda-migrate-user.html#cognito-user-pools-lambda-trigger-syntax-user-migration
     */
    clientMetadata: Record<string, string> | undefined;
    /**
     * One or more key-value pairs containing the validation data in the user's sign-in request. You can pass this data to
     * your Lambda function by using the ClientMetadata parameter in the InitiateAuth and AdminInitiateAuth API actions.
     *
     * Source: https://docs.aws.amazon.com/cognito/latest/developerguide/user-pool-lambda-migrate-user.html#cognito-user-pools-lambda-trigger-syntax-user-migration
     */
    validationData: Record<string, string> | undefined;
}, User>;
interface UserMigrationServices {
    clock: Clock;
    cognitoClient: CognitoService;
    lambda: Lambda;
}
export declare const UserMigration: ({ lambda, cognitoClient, clock, }: UserMigrationServices) => UserMigrationTrigger;
export {};
