var __create = Object.create;
var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  __markAsModule(target);
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && key !== "default")
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toModule = (module2) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", module2 && module2.__esModule && "default" in module2 ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
__export(exports, {
  SignUp: () => SignUp
});
var uuid = __toModule(require("uuid"));
var import_errors = __toModule(require("../errors"));
var import_deliveryMethod = __toModule(require("../services/messageDelivery/deliveryMethod"));
var import_userPoolService = __toModule(require("../services/userPoolService"));
const deliverWelcomeMessage = async (ctx, code, clientId, user, userPool, messages, clientMetadata) => {
  const deliveryDetails = (0, import_deliveryMethod.selectAppropriateDeliveryMethod)(userPool.options.AutoVerifiedAttributes ?? [], user);
  if (!deliveryDetails && !userPool.options.AutoVerifiedAttributes) {
    return null;
  } else if (!deliveryDetails) {
    throw new import_errors.InvalidParameterError("User has no attribute matching desired auto verified attributes");
  }
  await messages.deliver(ctx, "SignUp", clientId, userPool.options.Id, user, code, clientMetadata, deliveryDetails);
  return deliveryDetails;
};
const SignUp = ({
  clock,
  cognito,
  messages,
  otp,
  triggers,
  config
}) => async (ctx, req) => {
  var _a;
  const userPool = await cognito.getUserPoolForClientId(ctx, req.ClientId);
  const existingUser = await userPool.getUserByUsername(ctx, req.Username);
  if (existingUser) {
    throw new import_errors.UsernameExistsError();
  }
  const attributes = (0, import_userPoolService.attributesInclude)("sub", req.UserAttributes) ? req.UserAttributes ?? [] : [{ Name: "sub", Value: uuid.v4() }, ...req.UserAttributes ?? []];
  let userStatus = "UNCONFIRMED";
  if (triggers.enabled("PreSignUp")) {
    const { autoConfirmUser, autoVerifyEmail, autoVerifyPhone } = await triggers.preSignUp(ctx, {
      clientId: req.ClientId,
      clientMetadata: req.ClientMetadata,
      source: "PreSignUp_SignUp",
      userAttributes: attributes,
      username: req.Username,
      userPoolId: userPool.options.Id,
      validationData: void 0
    });
    if (autoConfirmUser) {
      userStatus = "CONFIRMED";
    }
    const isEmailUsername = (_a = config.UserPoolDefaults.UsernameAttributes) == null ? void 0 : _a.includes("email");
    const hasEmailAttribute = (0, import_userPoolService.attributesInclude)("email", attributes);
    if (isEmailUsername && !hasEmailAttribute) {
      attributes.push({ Name: "email", Value: req.Username });
    }
    if ((isEmailUsername || hasEmailAttribute) && autoVerifyEmail) {
      attributes.push({ Name: "email_verified", Value: "true" });
    }
    if ((0, import_userPoolService.attributesInclude)("phone_number", attributes) && autoVerifyPhone) {
      attributes.push({ Name: "phone_number_verified", Value: "true" });
    }
  }
  const now = clock.get();
  const updatedUser = {
    Attributes: attributes,
    Enabled: true,
    Password: req.Password,
    RefreshTokens: [],
    UserCreateDate: now,
    UserLastModifiedDate: now,
    Username: req.Username,
    UserStatus: userStatus
  };
  const code = otp();
  const deliveryDetails = await deliverWelcomeMessage(ctx, code, req.ClientId, updatedUser, userPool, messages, req.ClientMetadata);
  await userPool.saveUser(ctx, __spreadProps(__spreadValues({}, updatedUser), {
    ConfirmationCode: code
  }));
  if (updatedUser.UserStatus === "CONFIRMED" && triggers.enabled("PostConfirmation")) {
    await triggers.postConfirmation(ctx, {
      clientId: req.ClientId,
      clientMetadata: req.ClientMetadata,
      source: "PostConfirmation_ConfirmSignUp",
      username: updatedUser.Username,
      userPoolId: userPool.options.Id,
      userAttributes: (0, import_userPoolService.attributesAppend)(updatedUser.Attributes, (0, import_userPoolService.attribute)("cognito:user_status", updatedUser.UserStatus))
    });
  }
  return {
    CodeDeliveryDetails: deliveryDetails ?? void 0,
    UserConfirmed: updatedUser.UserStatus === "CONFIRMED",
    UserSub: (0, import_userPoolService.attributeValue)("sub", updatedUser.Attributes)
  };
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  SignUp
});
